//
//  TSBaseOperationQueue.swift
//  AIRingtone
//
//  Created by 100Years on 2025/3/20.
//

import Foundation
import Combine
class TSBaseOperationQueue {
    
    private(set) var queue: OperationQueue = OperationQueue()
    
    private var activeOperations:[String:TSBaseOperation] = [:]
    // 存储每个操作的 AnyCancellable
    private var cancellables: [String: AnyCancellable] = [:]
    // 存储 KVO 观察者
    private var operationCountObservation: NSKeyValueObservation?
    
    @Published var isAvailability:Bool = true
    private func upDateAvailability(){
        if queue.operationCount <= queue.maxConcurrentOperationCount {
            isAvailability = true
        }
        isAvailability = false
    }
    
    init(maxConcurrentOperationCount: Int = 1) {
        queue.maxConcurrentOperationCount = maxConcurrentOperationCount
        
        // 监听 operationCount 的变化
           operationCountObservation = queue.observe(\.operationCount, options: [.new]) { [weak self] (queue, change) in
               guard let self = self else { return }
               if let newValue = change.newValue {
                   upDateAvailability()
               }
           }
    }
    
    // 获取或创建操作
    func getOperation<T: TSBaseOperation>(uuid: String) -> T {
        if let existingOperation = activeOperations[uuid] as? T {
            return existingOperation
        } else {
            let operation = T(uuid: uuid)
            activeOperations[uuid] = operation
            queue.addOperation(operation)
            cancellables[uuid] = operation.$operationStatePblished.sink { [weak self] state in
                guard let self = self else { return }
                
                switch state {
                case .finished(let finished):
                    if finished == true {
                        cancelOperations(uuid: uuid)
                    }
                case .cancelled(let cancelled):
                    if cancelled == true {
                        cancelOperations(uuid: uuid)
                    }
                default: break
                }
            }
            return operation
        }
    }
    
    /// 清理所有下载任务
    func cancelAllOperations() {
        queue.cancelAllOperations()
        activeOperations = [:]
    }
    
    /// 清理某个下载任务
    /// - Parameter url: 下载任务的 URL
    func cancelOperations(uuid: String) {
        if let operation = activeOperations[uuid] {
            operation.cancel()
            activeOperations.removeValue(forKey: uuid)
            cancellables.removeValue(forKey: uuid)
        }
    }
    
    func waitUntilAllOperationsAreFinished() {
        queue.waitUntilAllOperationsAreFinished()
    }

}
